#################################
# start with some simulated data
#################################

rm(list=ls())
set.seed(1231)
library(lme4)

# data set size
N <- 1000

# true relationships
b.1<-c(-0.2, 0.4, 0.7)
b.2<-c(-0.5, 0, 0.1)
g<-rnorm(10, mean=0, sd=1)

# group stratification
x1.g <- matrix(data=NA, nrow=3, ncol=10)
x2.g <- matrix(data=NA, nrow=3, ncol=10)
x1.g[1,] <- runif(10, min=0, max=0.3)
x2.g[1,] <- runif(10, min=0, max=0.3)
x1.g[2,] <- runif(10, min=0, max=1-x1.g[1,])
x2.g[2,] <- runif(10, min=0, max=1-x2.g[1,])
x1.g[3,] <- 1-x1.g[1,]-x1.g[2,]
x2.g[3,] <- 1-x2.g[1,]-x2.g[2,]

# data generation
group<-c()
x.1<-c()
x.2<-c()
y<-c()

for(i in 1:N){
  
  group[i]<-floor(runif(1, min=1, max=11))
  
  x.1[i] <- which(rmultinom(1, size=1, prob=x1.g[,group[i]])==1)
  x.2[i] <- which(rmultinom(1, size=1, prob=x2.g[,group[i]])==1)
  
  y[i] <- b.1[x.1[i]] + b.2[x.2[i]] + g[group[i]] + rnorm(1, mean=0, sd=0.5)
    
}


# the multilevel regression portion of the model
mr<-lmer(y ~ (1 | x.1) + (1 | x.2) + (1 | group))

b.1.hat<-ranef(mr)$x.1     # individual-level predictors
b.2.hat<-ranef(mr)$x.2
g.hat<-ranef(mr)$group     # group predictors
int<-fixef(mr)             # grand intercept


# the poststratification portion of the model

group.pred<-c()
for(i in 1:10){
  
  group.pred[i] <- int + t(x1.g[,i])%*%as.matrix(b.1.hat) + t(x2.g[,i])%*%as.matrix(b.2.hat) + g.hat[i,] 
  
}

# the truth
group.true<-c()
for(i in 1:10){
  
  group.true[i]<-t(x1.g[,i])%*%as.matrix(b.1) + t(x2.g[,i])%*%as.matrix(b.2) + g[i] 
  
}

plot(group.pred~group.true)
abline(0, 1, lty=2)





################################################################
## this is from Jonathan Kastellec's primer on MRP            ##
## http://www.princeton.edu/~jkastell/MRP_primer/mrp_script.R ##
################################################################


rm(list=ls(all=TRUE))

library("arm")
library("foreign")

#read in megapoll and attach
marriage.data <- read.dta("gay_marriage_megapoll.dta", convert.underscore = TRUE) 

#read in state-level dataset

Statelevel <- read.dta("state_level_update.dta",convert.underscore = TRUE)
Statelevel <- Statelevel[order(Statelevel$sstate.initnum),]

#read in Census data
Census <- read.dta("poststratification 2000.dta",convert.underscore = TRUE)
Census <- Census[order(Census$cstate),]
Census$cstate.initnum <-  match(Census$cstate, Statelevel$sstate)

#Create index variables

#At level of megapoll

marriage.data$race.female <- (marriage.data$female *3) + marriage.data$race.wbh# from 1 for white males to 6 for hispanic females
marriage.data$age.edu.cat <- 4 * (marriage.data$age.cat -1) + marriage.data$edu.cat# from 1 for 18-29 with low edu to 16 for 65+ with high edu
marriage.data$p.evang.full <- Statelevel$p.evang[marriage.data$state.initnum]# proportion of evangelicals in respondent's state
marriage.data$p.mormon.full <-Statelevel$p.mormon[marriage.data$state.initnum]# proportion of mormon's in respondent's state
marriage.data$p.relig.full <- marriage.data$p.evang.full + marriage.data$p.mormon.full# combined evangelical + mormom proportions
marriage.data$p.kerry.full <- Statelevel$kerry.04[marriage.data$state.initnum]# kerry's % of 2-party vote in respondent's state in 2004

#At census level (same coding as above for all variables)

Census$crace.female <- (Census$cfemale *3) + Census$crace.WBH 
Census$cage.edu.cat <- 4 * (Census$cage.cat -1) + Census$cedu.cat 
Census$cp.evang.full<-  Statelevel$p.evang[Census$cstate.initnum]
Census$cp.mormon.full <- Statelevel$p.mormon[Census$cstate.initnum]
Census$cp.relig.full <- Census$cp.evang.full + Census$cp.mormon.full
Census$cp.kerry.full <-  Statelevel$kerry.04[Census$cstate.initnum]


#run individual-level opinion model

individual.model <- glmer(formula = yes.of.all ~ (1|race.female) + (1|age.cat) 
                          + (1|edu.cat) + (1|age.edu.cat) + (1|state) + (1|region) + (1|poll) + p.relig.full 
                          + p.kerry.full,data=marriage.data, family=binomial(link="logit"))
display(individual.model)

#examine random effects and standard errors for race-female
ranef(individual.model)$race.female
se.ranef(individual.model)$race.female

#create vector of state ranefs and then fill in missing ones
state.ranefs <- array(NA,c(51,1))
dimnames(state.ranefs) <- list(c(Statelevel$sstate),"effect")
for(i in Statelevel$sstate){
  state.ranefs[i,1] <- ranef(individual.model)$state[i,1]
}
state.ranefs[,1][is.na(state.ranefs[,1])] <- 0 #set states with missing REs (b/c not in data) to zero


#create a prediction for each cell in Census data
cellpred <- invlogit(fixef(individual.model)["(Intercept)"]
                     +ranef(individual.model)$race.female[Census$crace.female,1]
                     +ranef(individual.model)$age.cat[Census$cage.cat,1]
                     +ranef(individual.model)$edu.cat[Census$cedu.cat,1]
                     +ranef(individual.model)$age.edu.cat[Census$cage.edu.cat,1]
                     +state.ranefs[Census$cstate,1]
                     +ranef(individual.model)$region[Census$cregion,1]   
                     +(fixef(individual.model)["p.relig.full"] *Census$cp.relig.full)
                     +(fixef(individual.model)["p.kerry.full"] *Census$cp.kerry.full)
)

#weights the prediction by the freq of cell                                       
cellpredweighted <- cellpred * Census$cpercent.state

#calculates the percent within each state (weighted average of responses)
statepred <- 100* as.vector(tapply(cellpredweighted,Census$cstate,sum))
statepred




########################
## end Kastellec code ##
########################
library(devtools)
install_github("mrp", "malecki", sub="mrpdata")
install_github("mrp", "malecki", sub="mrp")


#############################################################
## the following code is straight out of the MRP help file ##
#############################################################

## Load example data.
data(CCES.complete)

## Helper datasets for other US applications of MRP:
data(spmap.states) # projected US state map
data(mrp.census)   # census with common demo strata
data(mrp.regions)  # regions data.frame with DC separate

## To ensure matching of strata between poll and population,
## both should be factors with identical names and levels.
CCES.complete <- within (CCES.complete, {
  education <- factor(education,exclude=NA)
  female <- factor(sex=="Female", labels=c("Male","Female"), exclude=NA)
  race <- factor(race,exclude=NA)
  f.race <- interaction(female,race)
})

## Poll has four levels of education, so we need to combine
## the top two levels in the census data. We'll also go ahead
## and trim it down to just the variables used here.

mrp.census <- within(mrp.census,{
  age <- factor(age,exclude=NA,labels=c("18-29","30-44","45-64","65+"))
  education[education=="postgraduate"] <- "college graduate"
  education <- factor(education,exclude=NA)
  edu <- factor(education,exclude=NA,labels=c("< High School",
                                              "High School",
                                              "Some College",
                                              "Graduated College"))
  state <- factor(state,exclude=NA)
  race <- factor(race,exclude=NA)
  f.race <- interaction(sex,race)
})
mrp.census <- na.omit(mrp.census)

## Ready to run simple mrp with poll and population:
mrp.simple <- mrp(ban.gaymarr ~ state+age+education+race,
                  data=CCES.complete,
                  population=mrp.census,
                  pop.weights="weighted2004")
print(100*poststratify(mrp.simple, ~ education+age), digits=2)
## Not run: 
## Fit a fuller model, adding state-level predictors:
## This model is also used in the not-run example
## for plotting maps.
## Note: for the population.formula here, we remove 'age' to demonstrate
## the use of the arg. The more common scenario is to remove 'poll'
## when combining several polls -- each gets a varying intercept,
## but they're not included in poststratification.

mrp.statelevel <- mrp(ban.gaymarr~
                        state+f.race+age+education,
                      data=CCES.complete,
                      population=mrp.census, pop.weights="weighted2008",
                      formula.pop.update= .~.-age,
                      grouplevel.data.frames=list(Statelevel,
                                                  mrp.regions),
                      grouplevel.expressions=list(
                        expression(age.edu <- interaction(age,education)),
                        ## an ordered factor, we use a normalized
                        ## continuous z.age as the prior mean for
                        ## varying intercepts of the 'age' groups.
                        ## That is, the prior mean for
                        ## age cat 1 of 4 (18-29) becomes (-.58)
                        expression(z.age <- rescale(age)))
)

